@@ -94,11 +94,10 @@ def handle_response(res):
         logger.info(f'Response: {res.json()}')
         return res.json()
     elif res.status_code in range(400, 500):
-        logger.error(f'Bad request. Status code: {res.status_code}')
-        logger.error(f'Response: {res.json()}')
+        logger.error(f'Bad request. Status code: {res.status_code} | Response: {res.content}')
         return res.json()
     else:
-        logger.error(f'Bad request. Status code: {res.status_code}')
+        logger.error(f'Bad request. Status code: {res.status_code} | response {res.content}')
 
 
 def get(url, **kwargs):
@@ -125,6 +124,13 @@ def delete(url, **kwargs):
     return handle_response(res)
 
 
+def model_to_io_file(model, filename='model.pth'):
+    """Load model as BytesIO (creates file in RAM)"""
+    model_io = io.BytesIO()
+    torch.save(model, model_io)
+    return filename, model_io.getvalue()
+
+
 # ============================================================
 # USER DATA
 # ============================================================
@@ -153,15 +159,28 @@ def update_job_status(job, status, attach_log=False):
     # Update status
     job['status'] = status
 
+    # Pop file fields
+    job.pop('trained_model', None)
+    job.pop('log', None)
+
     # Attach logfile if requested
     if attach_log:
         files = {'log': open(logger.file_path)}
         return put(job['url'], data=job, files=files)
     else:
-        job.pop('log', None)
         return put(job['url'], json=job)
 
 
+def upload_trained_model(job, model):
+    """Upload train model to job"""
+    # Remove previous logs and model files
+    job.pop('log', None)
+    job.pop('trained_model', None)
+
+    files = {'trained_model': model_to_io_file(model)}
+    return put(job['url'], data=job, files=files)
+
+
 def get_submissions(job_url=None, current_round=None):
     """
     Gets a list of submissions made by the user requesting the list. Include job_url if you want to
@@ -182,29 +201,26 @@ def get_submissions(job_url=None, current_round=None):
     return get(SUBMISSIONS_URL, params=params)
 
 
-def post_submission(model, job_url, current_round):
+def post_submission(model, job_url, current_round, number_observations):
     """
     Submit a new version of your model.
     :param torch.Module model: Torch model object.
     :param str job_url: Job URL associated to the model trained.
     :param int current_round: Version of the trained model (rounds).
+    :param number_observations: Number of observations used to train this model.
     :return:
     """
     logger.info(f'Submitting model...')
 
-    # Save model to buffer
-    model_io = io.BytesIO()
-    torch.save(model, model_io)
-    model_file = ('model.pth', model_io.getvalue())
-
     # Append files to upload
     files = {
-        'model': model_file,
+        'model': model_to_io_file(model),
         'log': open(logger.file_path, 'rb')}
 
     data = {
         'job': job_url,
-        'round': current_round
+        'round': current_round,
+        'number_observations': number_observations
     }
     return post(SUBMISSIONS_URL, files=files, data=data)
 
